import time
try:
    from email.utils import formatdate
except ImportError:
    from email.Utils import formatdate
from AccessControl.PermissionRole import rolesForPermissionOn
from zope.lifecycleevent import IObjectModifiedEvent
from Products.CMFCore.interfaces import IActionSucceededEvent
from zope.globalrequest import getRequest
from collective.siteimprove.interfaces import ICollectiveSiteimproveLayer


def triggerSiteimproveRecheck(obj, event):
    """ Sets cookie 'flag' to indicate that a recheck needs to take place
    """
    if ICollectiveSiteimproveLayer.providedBy(getRequest()):
        recheck = False

        if IObjectModifiedEvent.providedBy(event) and \
                'Anonymous' in rolesForPermissionOn('View', event.object):
            # object saved and publically visible
            recheck = True
        elif IActionSucceededEvent.providedBy(event):
            # workflow transition
            recheck = True

        if recheck:
            request = getRequest()
            expiration_seconds = time.time() + (1 * 60 * 60)  # 1 hour from now
            expires = formatdate(expiration_seconds, usegmt=True)
            request.response.setCookie("SI-Published", 'True', path='/',
                                       expires=expires)



from collective.siteimprove.browser.controlpanel import ISiteimproveSchema
from plone.app.layout.viewlets import common as base
from plone.registry.interfaces import IRegistry
from zope.component import queryMultiAdapter, getUtility


class SIViewletCommon:

    def update_common(self):
        self.token = None
        self.recheck = False

        context_state = queryMultiAdapter((self.context, self.request),
                                          name=u'plone_context_state')
        self.canonical_url = context_state.canonical_object_url()

        # lookup token
        registry = getUtility(IRegistry)
        siteimprove_registry = registry.forInterface(ISiteimproveSchema, False)
        if siteimprove_registry:
            if getattr(siteimprove_registry, 'token', None):
                self.token = siteimprove_registry.token

            if getattr(siteimprove_registry, 'canonical_site_url', None):
                portal_state = queryMultiAdapter((self.context, self.request),
                                                 name=u'plone_portal_state')
                self.canonical_url = self.canonical_url.replace(
                    portal_state.portal_url().rstrip('/'),
                    siteimprove_registry.canonical_site_url.rstrip('/')
                )

        # check if a publish event proceeded this, if yes, inject site improve
        # recheck js code
        cookie = self.request.cookies.get("SI-Published", None)
        if cookie:
            self.recheck = True
            # clear the cookie
            self.request.response.expireCookie("SI-Published", path='/')


class SiteimproveJavascriptViewOnlyViewlet(base.ViewletBase, SIViewletCommon):
    """ This will render Siteimprove JavaScript load in <head>.
        <head> section is retrofitted only if the viewlet is enabled.
    """

    def update(self):
        super(SiteimproveJavascriptViewOnlyViewlet, self).update()
        self.view_only = True
        self.update_common()


class SiteimproveJavascriptViewlet(base.ViewletBase, SIViewletCommon):
    """ This will render Siteimprove JavaScript load in <head>.
        <head> section is retrofitted only if the viewlet is enabled.
    """

    def update(self):
        super(SiteimproveJavascriptViewlet, self).update()
        self.view_only = False
        self.update_common()



from AccessControl.PermissionRole import rolesForPermissionOn
from Products.Five.browser import BrowserView


class AnonCanView(BrowserView):
    """Returns true if anonymous user can view content"""

    def __call__(self):
        """Can we see this"""
        return 'Anonymous' in rolesForPermissionOn('View', self.context)


class UseDomainView(BrowserView):
    """  Returns true if the domain function is to be used in the
         siteimprove js header false implies that input function will
         be used instead
    """

    def __call__(self):
        # figure out if domain or input function should be used

        # if the view is an add or edit form use input
        # 'edit' covers: 'edit', 'base_edit','atct_edit','@@edit',
        #                adding archetype objects
        #                editing archetype and dexterity objects
        # 'add' covers:  adding dexterity objects
        filter_strings = ['edit', 'add']
        for x in filter_strings:
            if x in self.request.URL:
                return False  # use input

        # otherwise use domain
        return True



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.statusmessages.interfaces import IStatusMessage
from logging import getLogger
from plone.app.registry.browser import controlpanel
from zope.i18nmessageid import MessageFactory
from zope.interface import Interface
from zope import schema
from z3c.form import button
import requests

log = getLogger(__name__)


class ISiteimproveSchema(Interface):

    token = schema.TextLine(
        title=_(u'Token'),
        description=_(u'Configure Siteimprove Plugin token.'),
        required=True
    )

    canonical_site_url = schema.URI(
        title=_(u'Public Site URL'),
        description=_(u'Set this if your public site url is different, '
                      u'from the url used by content editors.'),
        required=False,
    )


class SiteimproveControlPanelForm(controlpanel.RegistryEditForm):

    id = "SiteimproveControlPanel"
    label = _(u"Siteimprove Settings")
    schema = ISiteimproveSchema
    formErrorsMessage = _('Error fetching token from siteimprove.')

    @button.buttonAndHandler(_(u"Save"), name='save')
    def handleSave(self, action):
        super(SiteimproveControlPanelForm, self).handleSave(self, action)

    @button.buttonAndHandler(_('Request new token'), name='token')
    def handleRequestNewToken(self, action):
        # get plone version
        core_versions = getToolByName(aq_inner(self.context),
                                      'portal_migration').coreVersions()
        version_string = 'Plone %s' % core_versions['Plone']

        # request new token
        try:
            response = requests.get("https://my2.siteimprove.com/auth/token",
                                    data={'cms': version_string})
            response_data = response.json()
            if response_data.get('token'):
                self.request.form['form.widgets.token'] = response_data['token']
        except (requests.exceptions.RequestException, ValueError):
            log.exception("Error fetching token from siteimprove.")

        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        self.applyChanges(data)
        IStatusMessage(self.request).addStatusMessage(
            _(u"Token received."),
            "info"
        )
        self.request.response.redirect(self.request.getURL())

    @button.buttonAndHandler(_(u"Cancel"), name='cancel')
    def handleCancel(self, action):
        super(SiteimproveControlPanelForm, self).handleCancel(self, action)

    def updateActions(self):
        super(SiteimproveControlPanelForm, self).updateActions()
        self.actions['token'].addClass("context")


class SiteimproveControlPanel(controlpanel.ControlPanelFormWrapper):
    form = SiteimproveControlPanelForm
